/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.controller;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.operations.IOperationHistory;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;

import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.Synonym;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.taxeditor.datasource.CdmTransactionController;
import eu.etaxonomy.taxeditor.editor.AbstractTaxonEditor;
import eu.etaxonomy.taxeditor.editor.MultiPageTaxonEditor;
import eu.etaxonomy.taxeditor.editor.TemporaryTaxon;
import eu.etaxonomy.taxeditor.editor.description.TaxonDescriptionEditor;
import eu.etaxonomy.taxeditor.editor.name.TaxonNameEditor;
import eu.etaxonomy.taxeditor.model.CdmSessionDataRepository;
import eu.etaxonomy.taxeditor.model.CdmUtil;
import eu.etaxonomy.taxeditor.model.NameEditorInput;
import eu.etaxonomy.taxeditor.navigation.RecentNamesView;
import eu.etaxonomy.taxeditor.navigation.TaxonomicTreeViewer;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;

/**
 * @author n.hoffmann
 * @created 20.01.2009
 * @version 1.0
 */
public class EditorController {
	private static final Logger logger = Logger.getLogger(EditorController.class);

	private static Map<Taxon, TaxonNameEditor> taxonNameEditors;
	private static Map<Taxon, TaxonDescriptionEditor> taxonDescriptionEditors;

	private static boolean isSaving = false;
	
	private static IWorkbenchPage getActivePage(){
		return GlobalController.getActivePage();
	}
	
	/**
	 * @param input
	 * @return
	 * @throws PartInitException
	 */
	private static IEditorPart getEditorByInput(IEditorInput input)
			throws PartInitException {
		for (IEditorReference reference : getActivePage().getEditorReferences()) {
			if (reference.getEditorInput().equals(input)) {
				IEditorPart editor = reference.getEditor(false);
				return editor;
			}
		}
		return null;
	}

	public static boolean closeAll() {
		for (IEditorPart editor : getOpenEditors()) {
			if (!getActivePage().closeEditor(editor, true)) {
				return false;
			}
		}
		return true;
	}

	public static IEditorPart getEditorByTaxon(Taxon taxon)
			throws PartInitException {
		IEditorInput input = new NameEditorInput(taxon);
		return getEditorByInput(input);
	}

	/**
	 * @param input
	 * @param editorId
	 * @return 
	 * @throws PartInitException
	 */
	private static IEditorPart open(IEditorInput input, String editorId)
			throws PartInitException {
		return getActivePage().openEditor(input, editorId);
	}

	/** 
	 * Open a taxon editor for the given taxon. If the taxon has been saved before, it is
	 * considered initialized, and opening its editor will give it an entry in the list
	 * of recent names and open its node in the tree. 
	 * 
	 * @param taxon
	 * @param isInitialized
	 * @return
	 */
	public static IEditorPart open(Taxon taxon, boolean isInitialized){
		
		if (isInitialized) {
			
			TaxonomicTreeViewer treeViewer = TreeController.getTreeViewer(); 
			if (treeViewer != null) {
				treeViewer.revealTaxon(taxon);
			}
			
			RecentNamesView.addRecentName(taxon);			
		}
		
		IEditorInput input = new NameEditorInput(taxon);
		
		try {
			return open(input, MultiPageTaxonEditor.ID);
		} catch (PartInitException e) {
			logger.error("Error opening editor.", e);
		}
		return null;
	}
	
	public static IEditorPart open(Taxon taxon){
		
//		TemporaryTaxon taxonCopy = TemporaryTaxon.newInstance(taxon);
		Taxon taxonCopy = getEditorTaxon(taxon);
		
		return open(getEditorTaxon(taxon), true);
	}
	
	/**
	 * @param taxon
	 * @return
	 */
	private static Taxon getEditorTaxon(Taxon taxon) {
		return (Taxon) CdmUtil.getTaxonService().getTaxonByUuid(taxon.getUuid());
	}

	/**
	 * An unitialized taxon is one that hasn't been saved yet. As such, it should appear in neither
	 * the list of recent names nor in the taxonomic tree when opened.
	 * 
	 * @param taxon
	 * @return
	 */
	public static IEditorPart openUnitializedChildTaxon(Taxon parentTaxon) {
		
		TaxonNameBase name = PreferencesUtil.getInstanceOfPreferredNameClass();
		Taxon taxon = Taxon.NewInstance(name, parentTaxon.getSec());
		parentTaxon.addTaxonomicChild(taxon, null, null);
		
		return open(taxon, false);
	}

	/**
	 * An unitialized taxon is one that hasn't been saved yet. As such, it should appear in neither
	 * the list of recent names nor in the taxonomic tree when opened.
	 * 
	 * @param taxon
	 * @return
	 */
	public static IEditorPart openUnitializedRootTaxon() {
		
		TaxonNameBase name = PreferencesUtil.getInstanceOfPreferredNameClass();
		Taxon taxon = Taxon.NewInstance(name, 
				CdmSessionDataRepository.getDefault().getDefaultSec());
		
		return open(taxon, false);
	}
	
	public static IEditorPart openTaxonEditor(TaxonBase taxonBase) {

		Taxon taxon = null;
		if (taxonBase instanceof Synonym) {
			// TODO: in case of pro parte synonym or any other where we might have multiple
			// accepted taxa we have to provide a mechanism that can deal with that
			// TODO set focus to the synonym
			taxon = (Taxon) ((Synonym) taxonBase).getAcceptedTaxa().toArray()[0];
		} else {
			taxon = (Taxon) taxonBase;
		}

		// Open a taxon editor
		return EditorController.open(taxon);
	}

	/**
	 * Redraws an open editor if it exists for the given taxon
	 * 
	 * @param taxon
	 * @return
	 */
	public static boolean redraw(Taxon taxon){
//		return redraw(EditorController.getNameEditor(taxon)) &&
//				redraw(EditorController.getDescriptionEditor(taxon));
		
		boolean returnCode = redraw(EditorController.getNameEditor(taxon)) &&
				redraw(EditorController.getDescriptionEditor(taxon));
		getActiveEditor().setFocus();
		return returnCode;
	}

	private static boolean redraw(AbstractTaxonEditor editor) {	
		if(editor == null || editor.redraw()){
			
//			if (getActiveEditor().setFocus())
//			getActivePage()
			
			// Mark editor as changed and unsaved
			editor.setDirty();	
			return true;	
		}	
		return false;		
	}
	
	/**
	 * @param taxon
	 * @param save
	 */
	public static void close(Taxon taxon, boolean save) {
		IEditorPart editor;
		try {
			editor = getEditorByTaxon(taxon);
			close(editor, save);
		} catch (PartInitException e) {
			logger.error("Error closing taxon editor", e);
		}
		
	}

	/**
	 * @param editor
	 * @param save
	 * @throws PartInitException
	 */
	private static void close(IEditorPart editor, boolean save)
			throws PartInitException {
		if (editor != null) {
			getActivePage().closeEditor(editor, save);
		}
	}

	public static boolean save(Taxon taxon, boolean confirm) {
		IEditorPart editor = null;
		try {
			editor = getEditorByTaxon(taxon);
		} catch (PartInitException e) {
			logger.error("Error saving taxon.", e);
		}
		if (editor != null) {
			return GlobalController.getActivePage().saveEditor(editor, confirm);
		}
		return false;
	}

	public static void saveAll(){
		setSaving(true);
		
		// Get all open windows
		for (IEditorPart taxonEditor : getOpenEditors()) {
	
			// Save the dirty ones
			if (taxonEditor.isDirty()) {
				
				
				IEditorInput input = taxonEditor.getEditorInput();
				if (input.getAdapter(Taxon.class) != null) {
					Taxon taxon = (Taxon) input.getAdapter(Taxon.class);
					CdmSessionDataRepository.getDefault().saveTaxon(taxon);
					if (taxonEditor instanceof MultiPageTaxonEditor) {
						((MultiPageTaxonEditor) taxonEditor).setDirtyExtern(false);
					}
				}
			}
		}
	
		// Commit the transaction	
		CdmTransactionController.commitTransaction();
	
		// Force library objects to be associated with new transaction
		CdmSessionDataRepository.getDefault().clearNonTaxonData();
		
		// Start a new transaction
		CdmTransactionController.startTransaction();
		
		// Put all open taxa in the new transaction
		CdmTransactionController.addSessionTaxaToTransaction();
		
		setSaving(false);
		// TODO: delete undoHistory
	}

	public static void setSaving(boolean isSaving) {
		EditorController.isSaving = isSaving;
	}

	public static boolean isSaving() {
		return isSaving;
	}

	public static void addNameEditor(
			Taxon taxon, TaxonNameEditor taxonNameEditor) {
		if (taxonNameEditors == null) {
			taxonNameEditors = new HashMap<Taxon, TaxonNameEditor>();
		}
		taxonNameEditors.put(taxon, taxonNameEditor);
	}

	public static TaxonNameEditor getNameEditor(Taxon taxon) {
		if (taxonNameEditors == null) {
			return null;
		}
		return taxonNameEditors.get(taxon);
	}

	public static void addDescriptionEditor(
			Taxon taxon, TaxonDescriptionEditor taxonDescriptionEditor) {
		if (taxonDescriptionEditors == null) {
			taxonDescriptionEditors = new HashMap<Taxon, TaxonDescriptionEditor>();
		}
		taxonDescriptionEditors.put(taxon, taxonDescriptionEditor);
	}

	public static TaxonDescriptionEditor getDescriptionEditor(Taxon taxon) {
		if (taxonDescriptionEditors == null) {
			return null;
		}
		return taxonDescriptionEditors.get(taxon);
	}
	
	/**
	 * Returns a set of all currently open 
	 * <code>MultiPageTaxonEditor</code>s.
	 * 
	 * @return
	 */
	public static Set<IEditorPart> getOpenEditors() {
		
		Set<IEditorPart> taxonEditors = new HashSet<IEditorPart>();
		
		for (IEditorReference reference : getActivePage().getEditorReferences()) {
			IEditorPart editor = reference.getEditor(false);
			if (editor instanceof MultiPageTaxonEditor) {
				taxonEditors.add(editor);
			}
		}
		return taxonEditors;
	}
	
	/**
	 * Returns the currently active taxon editor
	 * 
	 * @return the taxon editor that has focus
	 */
	public static MultiPageTaxonEditor getActiveEditor(){
		if(getActivePage().getActiveEditor() instanceof MultiPageTaxonEditor){
			return (MultiPageTaxonEditor) getActivePage().getActiveEditor();
		}
		return null;
	}

	public static IUndoContext getUndoContext(Taxon taxon) {
			// TODO make this taxon name editor specific
	//		return getTaxonNameEditor(taxon).getUndoContext();
			return getUndoContext();
	}

	public static IUndoContext getUndoContext() {
		return IOperationHistory.GLOBAL_UNDO_CONTEXT;
	}


	

}
