/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.datasource;

import java.util.ArrayList;

import org.apache.log4j.Logger;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CCombo;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.Text;

import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.database.CdmDataSource;
import eu.etaxonomy.cdm.database.CdmPersistentDataSource;
import eu.etaxonomy.cdm.database.DatabaseTypeEnum;
import eu.etaxonomy.cdm.database.ICdmDataSource;
import eu.etaxonomy.taxeditor.controller.GlobalController;
/**
 * List of database types comes from {@link eu.etaxonomy.cdm.database.DatabaseTypeEnum}.
 * 
 * @author p.ciardelli
 * @created 11.12.2008
 * @version 1.0
 */
public class CdmDataSourceDialog extends Dialog {
	private static final Logger logger = Logger.getLogger(CdmDataSourceDialog.class);
	
	private Text portText;
	private Text serverText;
	private Text passwordText;
	private Text databaseNameText;
	private Text datasourceNameText;
	private CCombo databaseTypeCombo;
	private Text usernameText;
	
	private Composite container;
	private Composite editDatasourceComposite;
	private Group authenticationGroup;
	private Group locationGroup;
	private Button saveButton;
	private Composite dbConfigComposite;

	private ArrayList<DatabaseTypeEnum> databaseTypes;
	private ICdmDataSource dataSource = null;

	private TableViewer dataSourceViewer;

	private Button btnEdit;

	private Button btnRemove;

	private ICdmDataSource selectedDataSource;

	private Shell parentShell;

	/**
	 * Any time user modifies datasource properties, force a save or a cancel
	 */
	ModifyListener modifyListener = new ModifyListener() {
		@Override
		public void modifyText(ModifyEvent e) {
			forceUserToSave(true);
		}
	};
	
	/**
	 * Create the dialog
	 * @param parentShell
	 */
	public CdmDataSourceDialog(Shell parentShell) {
		super(parentShell);
		this.parentShell = parentShell;
	}
	
	protected void configureShell(Shell shell) {
		super.configureShell(shell);
		shell.setText("Edit Data Sources");
	}
	
	/**
	 * Create contents of the dialog
	 * @param parent
	 */
	@Override
	protected Control createDialogArea(Composite parent) {
		
		// Create top-level composite 
		container = (Composite) super.createDialogArea(parent);
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 5;
		container.setLayout(gridLayout);

		// Create buttons to edit list
		btnEdit = new Button(container, SWT.NONE);
		btnEdit.setText("Edit ...");
		btnEdit.setEnabled(false);
		btnEdit.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
//				if (getSelectedTypeDesignation() != null) {	
//					createTypeDesignationWizard(getSelectedTypeDesignation());
//				}
				if (getSelectedDataSource() != null) {
					showDatasource(getSelectedDataSource());
				}
			}
		});

		btnRemove = new Button(container, SWT.NONE);
		btnRemove.setText("Remove");
		btnRemove.setEnabled(false);
		btnRemove.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				if (getSelectedDataSource() != null) {
					removeDataSource(getSelectedDataSource());
					setSelectedDataSource(null);
				}
			}
		});
		
		final Button btnAdd = new Button(container, SWT.NONE);
		final GridData gd_btnAdd = new GridData(SWT.RIGHT, SWT.CENTER, true,
				false);
		btnAdd.setLayoutData(gd_btnAdd);
		btnAdd.setText("Add ...");
		btnAdd.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				showNewDatasource();
			}
		});
		
		// Create viewer for datasource list
		dataSourceViewer = new TableViewer(container, SWT.BORDER);
		GridData viewerLayout = new GridData(SWT.LEFT, SWT.FILL, false, true, 3, 1);
		viewerLayout.widthHint = 162;
		dataSourceViewer.getTable().setLayoutData(viewerLayout);
		
		// Set providers for datasource list
		dataSourceViewer.setContentProvider(new ArrayContentProvider());
		dataSourceViewer.setLabelProvider(new LabelProvider() {
			public String getText(Object element) {
				if (element instanceof ICdmDataSource) {
					return ((ICdmDataSource) element).getName();
				} 
				return super.getText(element);
			}
		});
		
		// Add data sources to viewer		
		refreshDataSourceViewer();
		
		// Double-clicking on a dataSource in the viewer puts it in the editable area
		dataSourceViewer.addDoubleClickListener(new IDoubleClickListener() {
			public void doubleClick(DoubleClickEvent event) {
				
				Object dataSource = 
						((IStructuredSelection) event.getSelection()).
						getFirstElement();
				
				if (dataSource instanceof ICdmDataSource) {
					
					// Existing datasource
					showDatasource((ICdmDataSource) dataSource);
				} else {
					
					// New datasource
					showDatasource(null);
				}
			}
		});
		
		// Create "remove datasource" rt-click menu
		Table dataSourceControl = dataSourceViewer.getTable();
		final MenuManager manager = new MenuManager();
		final Menu menu = manager.createContextMenu(dataSourceControl);
		dataSourceControl.setMenu(menu);
		
		// Add right-click menu to delete datasources
		dataSourceControl.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {

				setEnableTypeDesignationButtons(true);
				
				// Get content of widget's data field
				if (e.item.getData() instanceof ICdmDataSource) {
					setSelectedDataSource((ICdmDataSource) e.item.getData());					
				}
				
				
				// Clear menu
//				manager.removeAll();
					
				// Only persistent data sources can be removed -
				//	H2 database and "New datasource" entry remain untouched
//				if (data instanceof CdmPersistentDataSource) {
//					manager.add(new RemoveDataSourceAction((CdmPersistentDataSource) data));
//				}
			}
		});

		// Create editDatasourceComposite to display a dataSource's name and type
		editDatasourceComposite = new Composite(container, SWT.NONE);
		GridData datasourceGridData = new GridData(SWT.FILL, SWT.TOP, true, true);
		editDatasourceComposite.setLayoutData(datasourceGridData);
		GridLayout datasourceLayout = new GridLayout();
		datasourceLayout.numColumns = 2;
		editDatasourceComposite.setLayout(datasourceLayout);
		
		// Create label and input for dataSource name
		Label datasourceNameLabel = new Label(editDatasourceComposite, SWT.NONE);
		datasourceNameLabel.setText("Datasource Name:");
		datasourceNameText = new Text(editDatasourceComposite, SWT.BORDER);
		datasourceNameText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

		// Create label and dropdown for database type		
		Label databaseTypeLabel = new Label(editDatasourceComposite, SWT.NONE);
		databaseTypeLabel.setText("Database Type:");
		databaseTypeCombo = new CCombo(editDatasourceComposite, SWT.BORDER);
		GridData comboLayout = new GridData(SWT.FILL, SWT.CENTER, false, false);
		databaseTypeCombo.setLayoutData(comboLayout);
		populateComboBoxItems();	
		
		// Create listener to display database type-specific config options
		databaseTypeCombo.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				
				// Get database type at the selected index
				DatabaseTypeEnum type = databaseTypes.get(databaseTypeCombo.getSelectionIndex());
				
				// Display type-specific input fields
				showDatabaseConfig(dataSource, type);
			}
		});
		
		// Hide editable fields until a dataSource is chosen
		editDatasourceComposite.setVisible(false);
		
		return container;
	}

	/**
	 * 
	 */
	protected void showNewDatasource() {
		showDatasource(null);
	}

	/**
	 * @param selectedDataSource
	 */
	protected void removeDataSource(ICdmDataSource dataSource) {
		
		if (dataSource instanceof CdmPersistentDataSource) {
		
			// Delete from persistent data sources
			CdmDataSourceRepository.getDefault().delete((CdmPersistentDataSource) dataSource);
		
			// Refresh list of data sources to reflect changes
			refreshDataSourceViewer();
		
		} else {
			MessageDialog.openInformation(parentShell, "Data source not persistent", 
					"This data source cannot be removed. Only persistent data sources can be removed.");
		}
	}

	/**
	 * @return
	 */
	protected ICdmDataSource getSelectedDataSource() {
		return selectedDataSource;
	}

	/**
	 * 
	 */
	protected void setSelectedDataSource(ICdmDataSource selectedDataSource) {
		this.selectedDataSource = selectedDataSource;
	}

	/**
	 * @param enabled
	 */
	protected void setEnableTypeDesignationButtons(boolean enabled) {
		btnEdit.setEnabled(enabled);
		btnRemove.setEnabled(enabled);
	}

	private void showDatasource(ICdmDataSource dataSource) {
		
		// Put dataSource in field
		this.dataSource = dataSource;
		
		// Make editable fields visible
		editDatasourceComposite.setVisible(true);
		
		// Remove any existing db config data
		removeDbConfigFields();
		
		// Populate name and database type if this is a non-null dataSource
		if (dataSource != null) {
			datasourceNameText.setText(dataSource.getName());
			datasourceNameText.setEnabled(false);
			selectDatabaseType(dataSource.getDatabaseType());
			databaseTypeCombo.setEnabled(false);
		} else {
			datasourceNameText.setText("");
			datasourceNameText.setEnabled(true);
			databaseTypeCombo.setText(""); // for some reason, .deselectAll and .clearSelection don't work as advertised
			databaseTypeCombo.setEnabled(true);
		}
		
	}
	
	private void showDatabaseConfig(ICdmDataSource dataSource, DatabaseTypeEnum type) {
		if (type.equals(DatabaseTypeEnum.MySQL)) {
			createMySqlForm(dataSource);
		} else {
			// TODO Intrusive - change to text
			MessageDialog.openInformation(GlobalController.getShell(), "Database not yet implemented", 
					"Creation or editing of datasources using a(n) '" + type.getName() + "' database not yet implemented.");
		}
	}
	
	private void selectDatabaseType(DatabaseTypeEnum type) {
		
		int i = 0;
		
		// Go through types list until match is found
		for (DatabaseTypeEnum typeInList : databaseTypes ) {
			if (typeInList.equals(type)) {
				
				// Set combo to the type
				databaseTypeCombo.select(i);
				break;
			}
			i++;
		}
		
		// Display type-specific input fields
		showDatabaseConfig(dataSource, type);
		
//		// If we're this far, database type was not in the list 
//		MessageDialog.openInformation(UiUtil.getShell(), "Database not yet implemented", 
//				"This dataSource expects a database type '" + type.getName() + 
//				"', which is not in the Taxonomic Editor's list of expected databases.");
	}
	
	private void populateComboBoxItems() {
		
		// Init DB types
		if (databaseTypes == null) {
			databaseTypes = new ArrayList<DatabaseTypeEnum>();
		}
		
		// Add types to the type drop-down and to the types collection
		for (DatabaseTypeEnum type : DatabaseTypeEnum.getAllTypes()) {
			databaseTypeCombo.add(type.getName());
			databaseTypes.add(type);
		}
	}

	/**
	 * Create contents of the button bar
	 * @param parent
	 */
	@Override
	protected void createButtonsForButtonBar(Composite parent) {
		createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL,
				true);
		createButton(parent, IDialogConstants.CANCEL_ID,
				IDialogConstants.CANCEL_LABEL, false);
	}

	/**
	 * Return the initial size of the dialog
	 */
	@Override
	protected Point getInitialSize() {
		return new Point(500, 375);
	}

	private void removeDbConfigFields() {
		// Init db config area by removing all children
		if (dbConfigComposite != null) {
			for (Control child : dbConfigComposite.getChildren()) {
				child.dispose();
			}
		}
	}
	
	private void createH2Form() {
		
	}
	
	private void createMySqlForm(final ICdmDataSource dataSource) {

		// Create composite for db config as necessary
		if (dbConfigComposite == null) {
			dbConfigComposite = new Composite(editDatasourceComposite, SWT.NONE);
			dbConfigComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 2, 1));
			GridLayout formLayout = new GridLayout();
			formLayout.numColumns = 2;
			dbConfigComposite.setLayout(formLayout);
		}
		
		// Create group composite for authentication data
		authenticationGroup = new Group(dbConfigComposite, SWT.NONE);
		authenticationGroup.setText("Authentication");
		authenticationGroup.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 2, 1));
		GridLayout authenticationLayout = new GridLayout();
		authenticationLayout.numColumns = 2;
		authenticationGroup.setLayout(authenticationLayout);

		// Create database name label
		Label databaseNameLabel = new Label(authenticationGroup, SWT.NONE);
		databaseNameLabel.setText("Database Name:");

		// Create database name input
		databaseNameText = new Text(authenticationGroup, SWT.BORDER);
		databaseNameText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

		// Create username label
		Label usernameLabel = new Label(authenticationGroup, SWT.NONE);
		usernameLabel.setText("User Name:");

		// Create username input
		usernameText = new Text(authenticationGroup, SWT.BORDER);
		usernameText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		
		// Create password label
		Label passwordLabel = new Label(authenticationGroup, SWT.NONE);
		passwordLabel.setText("Password:");

		// Create password input
		passwordText = new Text(authenticationGroup, SWT.BORDER);
		passwordText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

//		// Create save password label
//		Label savePasswordLabel = new Label(authenticationGroup, SWT.NONE);
//		savePasswordLabel.setText("Save password:");
//
//		// Create save password checkbox
//		Button button = new Button(authenticationGroup, SWT.CHECK);

		// Create group composite for location data 
		locationGroup = new Group(dbConfigComposite, SWT.NONE);
		locationGroup.setText("Location");
		locationGroup.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 2, 1));
		GridLayout locationLayout = new GridLayout();
		locationLayout.numColumns = 2;
		locationGroup.setLayout(locationLayout);

		// Create host label
		Label serverLabel = new Label(locationGroup, SWT.NONE);
		serverLabel.setText("Host:");

		// Create host input
		serverText = new Text(locationGroup, SWT.BORDER);
		serverText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

		// Create port label
		Label portLabel = new Label(locationGroup, SWT.NONE);
		portLabel.setText("Port:");

		// Create port input
		portText = new Text(locationGroup, SWT.BORDER);
		portText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

		// Create composite for buttons
		Composite buttonComposite = new Composite(dbConfigComposite, SWT.NONE);
		buttonComposite.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, true, false));
		GridLayout buttonLayout = new GridLayout();
		buttonLayout.numColumns = 2;
		buttonComposite.setLayout(buttonLayout);
		
		// Create test connection button
		Button testButton = new Button(buttonComposite, SWT.NONE);
		testButton.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false));
		testButton.setText("Test connection");
		
		// Test connection when button is pressed
		testButton.addMouseListener(new MouseAdapter() {
			public void mouseUp(MouseEvent e) {
				testDbConfig();
			}
		});
		
		// Create save button
		saveButton = new Button(buttonComposite, SWT.NONE);
		GridData saveButtonLayout = new GridData(SWT.RIGHT, SWT.CENTER, false, false);
		saveButtonLayout.widthHint = 76;
		saveButton.setLayoutData(saveButtonLayout);
		saveButton.setText("Save");
		saveButton.setEnabled(false);
		
		// Save data to datasource on mouse click
		saveButton.addMouseListener(new MouseAdapter() {
			public void mouseUp(MouseEvent e) {
				saveDbConfig();
			}
		});
				
		// Populate fields with datasource data
		if (dataSource != null) {
			databaseNameText.setText(CdmUtils.Nz(dataSource.getDatabase()));
			passwordText.setText(CdmUtils.Nz(dataSource.getPassword()));
			usernameText.setText(CdmUtils.Nz(dataSource.getUsername()));
			serverText.setText(CdmUtils.Nz(dataSource.getServer()));
			portText.setText(String.valueOf(dataSource.getPort()));			
		}
		
		// Listen for changes to any input fields in order to toggle Save / OK
		databaseNameText.addModifyListener(modifyListener);
		usernameText.addModifyListener(modifyListener);
		passwordText.addModifyListener(modifyListener);
		serverText.addModifyListener(modifyListener);
		portText.addModifyListener(modifyListener);
		forceUserToSave(false);
		
		// Redraw
		container.layout(true, true);
	}

	/**
	 * @param mustSave
	 */
	private void forceUserToSave(boolean mustSave) {
		saveButton.setEnabled(mustSave);
		getButton(IDialogConstants.OK_ID).setEnabled(!mustSave);
	}

	private void testDbConfig() {
		
		ICdmDataSource dataSource = CdmDataSource.NewMySqlInstance(
				serverText.getText(),
				databaseNameText.getText(),
				getPort(), 
				usernameText.getText(), 
				passwordText.getText());
		if (dataSource.testConnection()) {
			MessageDialog.openConfirm(GlobalController.getShell(), "Test successful", "Test successful");
		} else {
			MessageDialog.openWarning(GlobalController.getShell(), "Test unsuccessful", "Test unsuccessful");			
		}
	}

	/**
	 * @return
	 */
	private int getPort() {
		try {
			return Integer.valueOf(portText.getText());
		} catch (NumberFormatException e) {
			portText.setText("");
			return -1;
		}
	}

	private void saveDbConfig() {
		
		// Get field values
		DatabaseTypeEnum databaseType = databaseTypes.get(databaseTypeCombo.getSelectionIndex());
		String dataSourceName = datasourceNameText.getText();
		String server = serverText.getText();
		String database = databaseNameText.getText();
		String username = usernameText.getText(); 
		String password = passwordText.getText();
		Integer port = getPort(); // new Integer(portText.getText());
		
		// Remove data source in case it already exists
		if (dataSource != null) {
			removeDataSource(dataSource);
		}
		
		if (port > 0) {
			// Save with port
			CdmDataSourceRepository.getDefault().save (
					dataSourceName, 
					databaseType, 
					server, 
					database, 
					port, 
					username, 
					password);			
		} else {
			// Save without port
			CdmDataSourceRepository.getDefault().save (
					dataSourceName, 
					databaseType, 
					server, 
					database,
					username, 
					password);			
		}
		
		// Refresh list of data sources to reflect changes
		refreshDataSourceViewer();
		forceUserToSave(false);
	}
	
	private void refreshDataSourceViewer() {
		
		// Populate the list with saved datasources and an entry for "new dataSource"
		dataSourceViewer.setInput(
				CdmDataSourceRepository.getDefault().getAll());
	}
	
	public boolean close() {
		return super.close();
	}
}
