/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy 
 * http://www.e-taxonomy.eu
 * 
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */
package eu.etaxonomy.taxeditor.navigation;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Set;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.Assert;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;

import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.model.CdmSessionDataRepository;

/**
 * @author p.ciardelli
 * @created 27.06.2008
 * @version 1.0
 */
public class TaxonomicTreeContentProvider implements ITreeContentProvider {
	private static final Logger logger = Logger
			.getLogger(TaxonomicTreeContentProvider.class);

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.ITreeContentProvider#getChildren(java.lang.Object)
	 */
	public Object[] getChildren(Object parentElement) {
		
		String errorMessage = "getChildren(parentElement) called with non-Taxon element.";
		
		// Make sure we are dealing with taxon objects
		Assert.isTrue(
				parentElement instanceof Taxon || parentElement instanceof Collection,
				errorMessage);
		
		// Return children for a single taxon
		if (parentElement instanceof Taxon) {
			return getTaxonChildren((Taxon) parentElement).toArray();
		}

		// Return children for a collection of taxa
		List<Taxon> children = new ArrayList<Taxon>();
		for (Object parent : (Collection) parentElement) {

			Assert.isTrue(parent instanceof Taxon, errorMessage);
			children.addAll(getTaxonChildren((Taxon) parent));
		}
		return children.toArray();
	}

	private Set<Taxon> getTaxonChildren(Taxon parentTaxon) {
		Set<Taxon> children = 
				CdmSessionDataRepository.getDefault().getTaxonomicChildren(parentTaxon);
	
		// BUG - Somehow when a child taxon 1) is added with "add quick name", 2) is the
		//	parent taxon's only child, and 3) is then deleted, the QuickNameTaxon singleton
		//	shows up in the parent taxon's children after it is deleted and therefore pops
		//	back up in the tree
		children.remove(QuickNameTaxon.getInstance());
				
		if (isQuickNameParent(parentTaxon)) {
			children.add(QuickNameTaxon.getInstance());
		}
		
		return children;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.ITreeContentProvider#getParent(java.lang.Object)
	 */
	public Object getParent(Object element) {
		
		Assert.isTrue(element instanceof Taxon, 
				"getParent(element) called with non-Taxon element.");
		
		return ((Taxon) element).getTaxonomicParent();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.ITreeContentProvider#hasChildren(java.lang.Object)
	 */
	public boolean hasChildren(Object element) {
		Assert.isTrue(element instanceof Taxon, 
				"hasChildren(element) called with non-Taxon element.");
		
		Taxon taxon = (Taxon) element;
		
		if (isQuickNameParent(taxon)) {
			return true;
		} else {
			return taxon.getTaxonomicChildrenCount() > 0;
		}
	}

	public boolean isQuickNameParent(Taxon taxon) {
		return taxon.equals(QuickNameTaxon.getInstance().getTaxonomicParent());
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
	 */
	public Object[] getElements(Object inputElement) {		
		return CdmSessionDataRepository.getDefault().getRootTaxa().toArray();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.IContentProvider#dispose()
	 */
	public void dispose() {}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer,
	 *      java.lang.Object, java.lang.Object)
	 */
	public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {
		// TODO Auto-generated method stub
	}
}