/**
* Copyright (C) 2009 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.propertysheet.type.wizard;

import org.apache.log4j.Logger;
import org.eclipse.core.databinding.observable.list.WritableList;
import org.eclipse.core.runtime.Assert;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import eu.etaxonomy.cdm.model.name.NameTypeDesignation;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.name.TypeDesignationBase;
import eu.etaxonomy.cdm.model.reference.Generic;
import eu.etaxonomy.cdm.model.reference.ReferenceBase;
import eu.etaxonomy.taxeditor.TaxEditorPlugin;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;
import eu.etaxonomy.taxeditor.propertysheet.reference.IReferenceSearch;
import eu.etaxonomy.taxeditor.propertysheet.reference.ReferenceSearchDialog;

/**
 * @author p.ciardelli
 * @created 09.03.2009
 * @version 1.0
 */
public class ChooseNameTypeWizardPage extends WizardPage {
	private static Logger logger = Logger
			.getLogger(ChooseNameTypeWizardPage.class);

	private TaxonNameBase name;	
	private NameTypeDesignation typeDesignation;
	private TaxonNameBase savedName;
	
	private ReferenceBase savedReference;
	
	private Text txtName;
	private Button btnClearName;
	private Text txtReference;
	private Button btnClearReference;
	private Button btnRejected;
	private Button btnConserved;
	private Button btnLecto;
	private Button btnNotDesignated;

	private WritableList typeDesignationsList;

	/**
	 * @param typeDesignation
	 * @param name
	 * @param typeDesignationsList 
	 */
	public ChooseNameTypeWizardPage(TypeDesignationBase typeDesignation,
			TaxonNameBase name, WritableList typeDesignationsList) {
		super("");
		
		Assert.isTrue(typeDesignation == null || typeDesignation instanceof NameTypeDesignation);
		
		this.typeDesignation = (NameTypeDesignation) typeDesignation;
		this.name = name;
		this.typeDesignationsList = typeDesignationsList;
		
		setTitle("Create or edit type designation");
		setDescription("Create or edit type designation for '" + name.getTitleCache() + "\".");
	}
	
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	public void createControl(Composite parent) {
		
		// Create composite for popup dialog	
		Composite container = new Composite(parent, SWT.NULL);
		container.setLayout(new GridLayout());
		setControl(container);
		
		// Create name text
		final Label lblName = new Label(container, SWT.NONE);
		lblName.setText("Choose a name either by searching or entering it as free text:");
		
		// Create 3-columned composite for name
		Composite nameComposite = new Composite(container, SWT.NULL);
		nameComposite.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		GridLayout gridLayout = new GridLayout();
		gridLayout.numColumns = 3;
		gridLayout.marginHeight = 0;
		gridLayout.marginWidth = 0;
		nameComposite.setLayout(gridLayout);
		
		// Create name input
		txtName = new Text(nameComposite, SWT.BORDER);
		txtName.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		txtName.addKeyListener(new KeyAdapter() {
			@Override
			public void keyReleased(KeyEvent e) {
				updatePage();
			}			
		});
		
		// Create name search button
		Button btnSearchName = new Button(nameComposite, SWT.NONE);
		btnSearchName.setEnabled(true);
		btnSearchName.setLayoutData(new GridData());
		btnSearchName.setText("Search ...");
		btnSearchName.addSelectionListener(new SelectionAdapter() {
			
			// Popup reference search
			public void widgetSelected(SelectionEvent e) {
				popupNameSearch();
			}
		});

		// Create clear name button
		btnClearName = new Button(nameComposite, SWT.NONE);
		btnClearName.setEnabled(false);
		btnClearName.setText("Clear");
		btnClearName.addSelectionListener(new SelectionAdapter() {
			
			// Clear selected reference
			public void widgetSelected(SelectionEvent e) {
				clearName();
			}
		});

		// Set text to name text if exists, disable ability to edit name
		if (typeDesignation != null && typeDesignation.getTypeName() != null) {
			
			savedName = typeDesignation.getTypeName();
			txtName.setText(savedName.getTitleCache());
			txtName.setEditable(false);
						
			btnClearName.setEnabled(false);
			btnSearchName.setEnabled(false);
		}
		
		// Tell user when he is entering a new name
		Label lblNewNameFeedback = new Label(container, SWT.NONE);
		lblNewNameFeedback.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, true, false));
		lblNewNameFeedback.setFont(
				TaxEditorPlugin.getDefault().italicizeFont(lblNewNameFeedback.getFont()));
		lblNewNameFeedback.setText("Existing names can not be edited.");
		
		// Create 2-columned composite for name
		Composite booleansComposite = new Composite(container, SWT.NULL);
		booleansComposite.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		GridLayout gridLayout2 = new GridLayout();
		gridLayout2.numColumns = 2;
		gridLayout2.marginHeight = 15;
		gridLayout2.marginWidth = 0;
		booleansComposite.setLayout(gridLayout2);
		
		btnRejected = new Button(booleansComposite, SWT.CHECK);
		btnRejected.setText("Is rejected type");
		btnRejected.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		
		btnConserved = new Button(booleansComposite, SWT.CHECK);
		btnConserved.setText("Is conserved type");
		btnConserved.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		
		btnLecto = new Button(booleansComposite, SWT.CHECK);
		btnLecto.setText("Is lecto type");
		btnLecto.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		
		btnNotDesignated = new Button(booleansComposite, SWT.CHECK);
		btnNotDesignated.setText("Is not designated");
		btnNotDesignated.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		
		// Set any boolean values
		if (typeDesignation != null) {
			if (typeDesignation.isRejectedType()) {
				btnRejected.setSelection(true);
			}
			if (typeDesignation.isConservedType()) {
				btnConserved.setSelection(true);
			}
			if (typeDesignation.isLectoType()) {
				btnLecto.setSelection(true);
			}
			if (typeDesignation.isNotDesignated()) {
				btnNotDesignated.setSelection(true);
			}
		}
		
		// Create reference text
		final Label lblReference = new Label(container, SWT.NONE);
		lblReference.setText("Choose a reference either by searching or entering it as free text:");
		
		// Create 3-columned composite for reference
		Composite referenceComposite = new Composite(container, SWT.NULL);
		referenceComposite.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		GridLayout gridLayout3 = new GridLayout();
		gridLayout3.numColumns = 3;
		gridLayout3.marginHeight = 0;
		gridLayout3.marginWidth = 0;
		referenceComposite.setLayout(gridLayout3);
		
		// Create reference input
		txtReference = new Text(referenceComposite, SWT.BORDER);
		txtReference.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		
		// Create reference search button
		Button btnSearchReference = new Button(referenceComposite, SWT.NONE);
		btnSearchReference.setEnabled(true);
		btnSearchReference.setLayoutData(new GridData());
		btnSearchReference.setText("Search ...");
		btnSearchReference.addSelectionListener(new SelectionAdapter() {
			
			// Popup reference search
			public void widgetSelected(SelectionEvent e) {
				popupReferenceSearch();
			}
		});

		// Create clear reference button
		btnClearReference = new Button(referenceComposite, SWT.NONE);
		btnClearReference.setEnabled(false);
		btnClearReference.setText("Clear");
		btnClearReference.addSelectionListener(new SelectionAdapter() {
			
			// Clear selected reference
			public void widgetSelected(SelectionEvent e) {
				clearReference();
			}
		});

		// Set text to reference text if exists
		if (typeDesignation != null && typeDesignation.getCitation() != null) {
			
			savedReference = typeDesignation.getCitation();
			txtReference.setText(savedReference.getTitleCache());
			txtReference.setEditable(false);
						
			btnClearReference.setEnabled(true);
		}
		
		// Tell user when he is entering a new reference
		Label lblNewRefFeedback = new Label(container, SWT.NONE);
		lblNewRefFeedback.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, true, false));
		lblNewRefFeedback.setFont(
				TaxEditorPlugin.getDefault().italicizeFont(lblNewRefFeedback.getFont()));
		lblNewRefFeedback.setText("Existing references can only be edited in property sheet.");
	}

	/**
	 * 
	 */
	protected void popupNameSearch() {
		Dialog dialog = new NameSearchDialog(getShell());
        Object value = ((NameSearchDialog) dialog).open();
        
		if (value instanceof TaxonNameBase) {
			setSavedName((TaxonNameBase) value);
			updatePage();
		}
	}

	protected void popupReferenceSearch() {
		Dialog dialog = new ReferenceSearchDialog(getShell(), 
				IReferenceSearch.BIBREF);
        Object value = ((ReferenceSearchDialog) dialog).open();
        
		if (value instanceof ReferenceBase) {
			setSavedReference((ReferenceBase) value);
		}
	}
	
	/**
	 * @param value
	 */
	private void setSavedName(TaxonNameBase savedName) {
		
		this.savedName = savedName;
		
		txtName.setText(savedName.getTitleCache());
		txtName.setEditable(false);
		
		btnClearName.setEnabled(true);
	}
	
	/**
	 * @param value
	 */
	private void setSavedReference(ReferenceBase reference) {
		
		savedReference = reference;
		
		txtReference.setText(reference.getTitleCache());
		txtReference.setEditable(false);
		
		btnClearReference.setEnabled(true);
	}

	/**
	 * 
	 */
	protected void clearName() {
		savedName = null;
		
		txtName.setText("");
		txtName.setEditable(true);
		
		btnClearName.setEnabled(false);		
	}
	
	/**
	 * 
	 */
	protected void clearReference() {
		savedReference = null;
		
		txtReference.setText("");
		txtReference.setEditable(true);
		
		btnClearReference.setEnabled(false);		
	}
	
	@Override
	public boolean canFlipToNextPage() {
		return isPageComplete();
	}

	public boolean isPageComplete() {
		return (txtName.getText().length() > 0);
	}
	
	private void updatePage() {
		
		getWizard().getContainer().updateButtons();
	}
	
    public void setPageComplete(boolean complete) {
        super.setPageComplete(complete);
        
        if (complete) {
        	
        	TaxonNameBase typeName = null;
        	if (savedName != null) {
        		typeName = savedName;
        	} else {
        		if (!txtName.getText().equals("")) {
        			typeName = PreferencesUtil.getInstanceOfPreferredNameClass();
        			typeName.setTitleCache(txtName.getText());
        		}
        	}
        	
        	ReferenceBase citation = null;
        	if (savedReference != null) {
        		citation = savedReference;
        	} else {
        		if (!txtReference.getText().equals("")) {
        			citation = Generic.NewInstance();
        			citation.setTitleCache(txtReference.getText());
        		}
        	}
        	        	
        	if (typeDesignation == null) {
        		typeDesignation = new TemporaryNameTypeDesignation(typeName, citation, null,
        				null, btnRejected.getSelection(), btnConserved.getSelection(), btnNotDesignated.getSelection());
        		typeDesignation.setLectoType(btnLecto.getSelection()); 
        	} else {
        		typeDesignation.setRejectedType(btnRejected.getSelection());
        		typeDesignation.setConservedType(btnConserved.getSelection());
        		typeDesignation.setLectoType(btnLecto.getSelection());
        		typeDesignation.setNotDesignated(btnNotDesignated.getSelection());
        		typeDesignation.setCitation(citation);
        	}
        	
        	typeDesignationsList.remove(typeDesignation);
    		typeDesignationsList.add(typeDesignation);
        }
    }
    
	public class TemporaryNameTypeDesignation extends
			NameTypeDesignation {
		
		/**
		 * @param typeName
		 * @param citation
		 * @param object
		 * @param object2
		 * @param selection
		 * @param selection2
		 * @param selection3
		 */
//		public TemporaryNameTypeDesignation() {
//			super(null, null, null, null, false, false, false);
//			setLectoType(false);
//		}
		
		public TemporaryNameTypeDesignation(TaxonNameBase typeName, ReferenceBase citation, String citationMicroReference,
				String originalNameString, boolean isRejectedType, boolean isConservedType, boolean isNotDesignated) {
			super(typeName, citation, citationMicroReference, originalNameString, 
					isRejectedType, isConservedType, isNotDesignated);
		}
	}
}
